"use strict";
exports.__esModule = true;
var ReducerManager_1 = require("./ReducerManager");
var SagaManager_1 = require("./SagaManager");
var RefCountedManager_1 = require("./RefCountedManager");
var SagaComparer_1 = require("../Utils/SagaComparer");
function getModuleManager(sagaMiddleware) {
    var _dispatch = null;
    var _reducerManager;
    var _sagaManager = RefCountedManager_1.getRefCountedManager(SagaManager_1.getSagaManager(sagaMiddleware), SagaComparer_1.equals);
    var _addedModules = new Set();
    var _dispatchActions = function (moduleId, actions) {
        if (!actions) {
            return;
        }
        if (!_dispatch) {
            throw new Error("setDispatch should be called on ModuleManager before adding any modules.");
        }
        actions.forEach(_dispatch);
    };
    var _addSagas = function (sagas) {
        if (!sagas) {
            return;
        }
        _sagaManager.add(sagas);
    };
    var _removeSagas = function (sagas) {
        if (!sagas) {
            return;
        }
        _sagaManager.remove(sagas);
    };
    var _addReducers = function (reducerMap) {
        if (!reducerMap) {
            return;
        }
        if (!_reducerManager) {
            _reducerManager = ReducerManager_1.getRefCountedReducerManager(ReducerManager_1.getReducerManager(reducerMap));
        }
        else {
            for (var key in reducerMap) {
                _reducerManager.add(key, reducerMap[key]);
            }
        }
    };
    var _removeReducers = function (reducerMap) {
        if (!reducerMap || !_reducerManager) {
            return;
        }
        for (var key in reducerMap) {
            _reducerManager.remove(key);
        }
    };
    // Create reduce function which redirects to _reduers.reduce
    var _reduce = function (s, a) {
        if (_reducerManager) {
            return _reducerManager.reduce(s, a);
        }
        return (s || null);
    };
    return {
        getReducer: _reduce,
        setDispatch: function (dispatch) {
            _dispatch = dispatch;
        },
        getItems: function () { return []; },
        add: function (modulesToAdd) {
            if (!modulesToAdd || modulesToAdd.length === 0) {
                return;
            }
            modulesToAdd = modulesToAdd.filter(function (module) { return module; });
            var justAddedModules = [];
            modulesToAdd.forEach(function (module) {
                if (!_addedModules.has(module.id)) {
                    _addedModules.add(module.id);
                    _addReducers(module.reducerMap);
                    justAddedModules.push(module);
                }
            });
            _dispatch && _dispatch({ type: "@@Internal/ModuleManager/ReducerAdded" });
            // add the sagas and dispatch actions at the end so all the reducers are registered
            justAddedModules.forEach(function (module) {
                // Running the sagas after registering the reducers as they sagas themselve might dispatch actions
                // and we want all reducers to be registered before dispatching any actions
                _addSagas(module.sagas);
                _dispatchActions(module.id, module.initialActions);
            });
        },
        remove: function (modulesToRemove) {
            if (!modulesToRemove) {
                return;
            }
            modulesToRemove = modulesToRemove.filter(function (module) { return module; });
            modulesToRemove.forEach(function (module) {
                if (_addedModules.has(module.id)) {
                    _dispatchActions(module.id, module.finalActions);
                    _removeReducers(module.reducerMap);
                    _removeSagas(module.sagas);
                    _addedModules["delete"](module.id);
                    _dispatch && _dispatch({ type: "@@Internal/ModuleManager/ModuleRemoved" });
                }
            });
        }
    };
}
exports.getModuleManager = getModuleManager;
