"use strict";
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
var Registry_1 = require("../Utils/Registry");
var redux_saga_1 = require("redux-saga");
/**
 * WHAT IS A PLUGIN?
 * A plugin is a contributable part of your workflow. Plugins are First Party only.
 * The plugin and the host communicate via posting messages to each other using Channels.
 * Every experience can export a PluginRegistry where the plugins can be registered.
 * The host can send messages to given plugin using the manager, and can recieve messages from plugin in a callback.
 * The Plugins get two Channels one for receiving messages and other for sending them.
 *
 * The biggest diffrentiator of a Plugin vs a module loaded in the store is the communication.
 * The plugin coomunicates with the host using Channels rather than Actions thus avoiding unnecessary reducer/selectors invocations.
 */
exports.END_TYPE = "CHANNEL_IS_ENDED";
/**
 * Returns the plugin manager
 * @param moduleManager Module manager to add the plugin modules so they can participate in redux flow
 * @param pluginRegistry Registry containing the plugins (host puts to hostChannel, and plugin puts to plugin channel)
 * @param initialPlugins Ids of initial plugins to load
 * @param processMessageFromPlugin This will be called when plugin sends a message to the host
 */
function getPluginManager(moduleManager, pluginRegistry, initialPlugins, processMessageFromPlugin) {
    // create a channel registry to manage the channels
    var channelRegistry = new Registry_1.Registry();
    // these will hold the stop function for each plugin
    var stoppers = {};
    var startPlugin = function (pluginId) {
        var hostChannel_1, pluginChannel_1, listener_1, moduleFactory, mdle, moduleRemover_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!(pluginRegistry.isRegistered(pluginId) && !stoppers[pluginId])) return [3 /*break*/, 7];
                    return [4 /*yield*/, redux_saga_1.effects.call(redux_saga_1.channel)];
                case 1:
                    hostChannel_1 = _a.sent();
                    return [4 /*yield*/, redux_saga_1.effects.call(redux_saga_1.channel)];
                case 2:
                    pluginChannel_1 = _a.sent();
                    return [4 /*yield*/, redux_saga_1.effects.fork(processPluginMessages, pluginId, pluginChannel_1, processMessageFromPlugin)];
                case 3:
                    listener_1 = _a.sent();
                    // register the channel for the plugin in the registry
                    return [4 /*yield*/, redux_saga_1.effects.call(channelRegistry.register, pluginId + "-host", hostChannel_1)];
                case 4:
                    // register the channel for the plugin in the registry
                    _a.sent();
                    return [4 /*yield*/, redux_saga_1.effects.call(channelRegistry.register, pluginId, pluginChannel_1)];
                case 5:
                    _a.sent();
                    moduleFactory = pluginRegistry.get(pluginId);
                    mdle = moduleFactory(hostChannel_1, pluginChannel_1);
                    return [4 /*yield*/, redux_saga_1.effects.call([moduleManager, moduleManager.addModule], mdle)];
                case 6:
                    moduleRemover_1 = _a.sent();
                    stoppers[pluginId] = function () {
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0: 
                                // put the end message to the channel so all listners can exit
                                return [4 /*yield*/, hostChannel_1.put({ type: exports.END_TYPE })];
                                case 1:
                                    // put the end message to the channel so all listners can exit
                                    _a.sent();
                                    return [4 /*yield*/, pluginChannel_1.put({ type: exports.END_TYPE })];
                                case 2:
                                    _a.sent();
                                    // remove the module
                                    return [4 /*yield*/, redux_saga_1.effects.call([moduleRemover_1, moduleRemover_1.remove])];
                                case 3:
                                    // remove the module
                                    _a.sent();
                                    // remove the channel from the registry
                                    return [4 /*yield*/, redux_saga_1.effects.call([channelRegistry, channelRegistry.unregister], pluginId)];
                                case 4:
                                    // remove the channel from the registry
                                    _a.sent();
                                    return [4 /*yield*/, redux_saga_1.effects.call([channelRegistry, channelRegistry.unregister], pluginId + "-host")];
                                case 5:
                                    _a.sent();
                                    // cancel the listener
                                    return [4 /*yield*/, redux_saga_1.effects.cancel(listener_1)];
                                case 6:
                                    // cancel the listener
                                    _a.sent();
                                    return [2 /*return*/];
                            }
                        });
                    };
                    _a.label = 7;
                case 7: return [2 /*return*/];
            }
        });
    };
    var stopPlugin = function (pluginId) {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!stoppers[pluginId]) return [3 /*break*/, 2];
                    return [4 /*yield*/, redux_saga_1.effects.call(stoppers[pluginId])];
                case 1:
                    _a.sent();
                    delete stoppers[pluginId];
                    _a.label = 2;
                case 2: return [2 /*return*/];
            }
        });
    };
    return {
        startInitialPlugins: function () {
            var _i, initialPlugins_1, pluginId;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _i = 0, initialPlugins_1 = initialPlugins;
                        _a.label = 1;
                    case 1:
                        if (!(_i < initialPlugins_1.length)) return [3 /*break*/, 4];
                        pluginId = initialPlugins_1[_i];
                        return [4 /*yield*/, redux_saga_1.effects.fork(startPlugin, pluginId)];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3:
                        _i++;
                        return [3 /*break*/, 1];
                    case 4: return [2 /*return*/];
                }
            });
        },
        isStarted: function (pluginId) {
            return !!stoppers[pluginId];
        },
        startPlugin: startPlugin,
        stopPlugin: stopPlugin,
        /** uses for posting the message to the plugin */
        postMessage: function (pluginId, message) {
            var hostChannel;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!channelRegistry.isRegistered(pluginId)) return [3 /*break*/, 3];
                        return [4 /*yield*/, redux_saga_1.effects.call([channelRegistry, channelRegistry.get], pluginId + "-host")];
                    case 1:
                        hostChannel = _a.sent();
                        return [4 /*yield*/, hostChannel.put(message)];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3: return [2 /*return*/];
                }
            });
        },
        dispose: function () {
            var registeredPlugins, _i, registeredPlugins_1, pluginId;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        registeredPlugins = Object.keys(stoppers);
                        _i = 0, registeredPlugins_1 = registeredPlugins;
                        _a.label = 1;
                    case 1:
                        if (!(_i < registeredPlugins_1.length)) return [3 /*break*/, 4];
                        pluginId = registeredPlugins_1[_i];
                        return [4 /*yield*/, redux_saga_1.effects.call(stopPlugin, pluginId)];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3:
                        _i++;
                        return [3 /*break*/, 1];
                    case 4: return [2 /*return*/];
                }
            });
        }
    };
}
exports.getPluginManager = getPluginManager;
function processPluginMessages(pluginId, pluginChannel, processMessageFromPluginChannel) {
    var message;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                if (!true) return [3 /*break*/, 3];
                return [4 /*yield*/, redux_saga_1.effects.take(pluginChannel)];
            case 1:
                message = _a.sent();
                // stop listening the channel if END message is received        
                return [4 /*yield*/, redux_saga_1.effects.call(processMessageFromPluginChannel, pluginId, message)];
            case 2:
                // stop listening the channel if END message is received        
                _a.sent();
                if (message["type"] === exports.END_TYPE) {
                    return [3 /*break*/, 3];
                }
                return [3 /*break*/, 0];
            case 3: return [2 /*return*/];
        }
    });
}
