"use strict";
exports.__esModule = true;
var redux_1 = require("redux");
var redux_saga_1 = require("redux-saga");
var ModuleManager_1 = require("./Managers/ModuleManager");
var RefCountedManager_1 = require("./Managers/RefCountedManager");
function configureStore(initialState, context) {
    var initialModules = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        initialModules[_i - 2] = arguments[_i];
    }
    // setup the saga middleware
    var sagaMonitor = window["__SAGA_MONITOR_EXTENSION__"] || undefined;
    var sagaMiddleware = redux_saga_1["default"]({
        context: context,
        sagaMonitor: sagaMonitor
    });
    var composeEnhancers = redux_1.compose;
    var enhancer = composeEnhancers(redux_1.applyMiddleware(sagaMiddleware));
    var modules = RefCountedManager_1.getRefCountedManager(ModuleManager_1.getModuleManager(sagaMiddleware), function (a, b) { return a.id === b.id; });
    // Create store
    var store = redux_1.createStore(modules.getReducer, initialState, enhancer);
    modules.setDispatch(store.dispatch);
    var addModule = function () {
        var moduleToBeAdded = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            moduleToBeAdded[_i] = arguments[_i];
        }
        modules.add(moduleToBeAdded);
        return {
            remove: function () {
                modules.remove(moduleToBeAdded);
            }
        };
    };
    // Add the module manager to the context
    context["moduleManager"] = {
        addModule: addModule
    };
    store.addModule = addModule;
    store.dispose = function () {
        // get all added modules and remove them
        var allModules = modules.getItems();
        modules.remove(allModules);
    };
    store.addModule.apply(store, initialModules);
    return store;
}
exports.configureStore = configureStore;
